'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('./_virtual/_rollupPluginBabelHelpers.cjs');
var umiCore = require('@metaplex-foundation/umi-core');
var errors = require('./errors.cjs');

class DefaultProgramRepository {
  constructor(context) {
    _rollupPluginBabelHelpers.defineProperty(this, "programs", []);
    this.context = context;
  }
  has(identifier, clusterFilter = 'current') {
    const programs = this.all(clusterFilter);
    return typeof identifier === 'string' ? programs.some(p => p.name === identifier) : programs.some(p => umiCore.samePublicKey(p.publicKey, identifier));
  }
  get(identifier, clusterFilter = 'current') {
    const cluster = this.parseClusterFilter(clusterFilter);
    const programs = this.all(clusterFilter);
    const program = typeof identifier === 'string' ? programs.find(p => p.name === identifier) : programs.find(p => umiCore.samePublicKey(p.publicKey, identifier));
    if (!program) {
      throw new errors.ProgramNotRecognizedError(identifier, cluster);
    }
    return program;
  }
  all(clusterFilter = 'current') {
    const cluster = this.parseClusterFilter(clusterFilter);
    return cluster === '*' ? this.programs : this.programs.filter(program => program.isOnCluster(cluster));
  }
  add(program, overrides = true) {
    if (overrides) {
      this.programs.unshift(program);
    } else {
      this.programs.push(program);
    }
  }
  resolveError(error, transaction) {
    // Ensure the error as logs.
    if (!Array.isArray(error.logs) || error.logs.length === 0) return null;
    const logs = error.logs.join('\n');

    // Parse the instruction number.
    const instructionRegex = /Error processing Instruction (\d+):/;
    const instruction = error.message.match(instructionRegex)?.[1] ?? null;

    // Parse the error code.
    const errorCodeRegex = /Custom program error: (0x[a-f0-9]+)/i;
    const errorCodeString = logs.match(errorCodeRegex)?.[1] ?? null;
    const errorCode = errorCodeString ? parseInt(errorCodeString, 16) : null;

    // Ensure we could find an instruction number and an error code.
    if (instruction === null || errorCode === null) return null;

    // Get the program ID from the instruction in the transaction.
    const instructionNumber = parseInt(instruction, 10);
    const programIndex = transaction.message.instructions?.[instructionNumber]?.programIndex ?? null;
    const programId = programIndex ? transaction.message.accounts[programIndex] : null;

    // Ensure we were able to find a program ID for the instruction.
    if (!programId) return null;

    // Find a registered program if any.
    let program;
    try {
      program = this.get(programId);
    } catch (_programNotFoundError) {
      return null;
    }

    // Finally, resolve the error.
    const resolvedError = program.getErrorFromCode(errorCode, error);
    return resolvedError ?? new errors.ProgramErrorNotRecognizedError(program, error);
  }
  parseClusterFilter(clusterFilter) {
    return clusterFilter === 'current' ? this.context.rpc.getCluster() : clusterFilter;
  }
}

exports.DefaultProgramRepository = DefaultProgramRepository;
//# sourceMappingURL=DefaultProgramRepository.cjs.map
